/*
 * @COPYRIGHT@
 * 
 * x-kernel v3.2
 * 
 * Copyright (c) 1993,1991,1990  Arizona Board of Regents
 * 
 * @COPYRIGHT@
 *
 * $RCSfile: mkboot.c,v $
 *
 * HISTORY
 * $Log: mkboot.c,v $
 * Revision 1.6  1994/06/11  00:03:16  menze
 * Added copyright notice
 *
 * Revision 1.5  1994/03/01  20:17:10  davidm
 * Padding is now controlled via option -p and output file name extension
 * changed from .boot to .exec.
 *
 * Revision 1.4  1994/02/23  02:44:05  davidm
 * (main): Changed to use <sys/exec.h> instead of individual object file
 * 	headers.
 *
 * Revision 1.3  1994/02/23  02:23:04  davidm
 * (main): bootfile size must be a multiple of 512 byte blocks; otherwise
 * lan boot truncates file.
 *
 * Revision 1.2  1994/02/04  05:51:33  davidm
 * (main): initializations for "infp" and "outfp" added.
 *
 * Revision 1.1  1994/02/04  00:39:16  davidm
 * Initial revision
 *
 */
/*
 * Converts an ECOFF object file into a bootable file.  The object file
 * must be a OMAGIC file (i.e., data follows immediatly after text).
 * See DEC "Assembly Language Programmer's Guide" documentation for
 * details.
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <scnhdr.h>
#include <sys/exec.h>

/* bootfile size must be multiple of BLOCK_SIZE: */
#define BLOCK_SIZE	512


int
main(int argc, char *argv[])
{
    FILE *infp = 0;
    FILE *outfp = 0;
    char outname[1024];
    char buf[8192];
    int i, size, nxfer;
    int tot_size, pad, block_size = BLOCK_SIZE;
    int verbose = 0;
    struct exec exhdr;		/* includes file & aout header */

    if (argc < 2) {
	fprintf(stderr, "usage: %s file ...\n", argv[0]);
	exit(1);
    } /* if */

    for (i = 1; i < argc; ++i) {
	if (strcmp(argv[i], "-v") == 0) {
	    verbose = ~verbose;
	    continue;
	} /* if */

	if (strncmp(argv[i], "-p", 2) == 0) {
	    block_size = atoi(&argv[i][2]);
	    if (block_size < 0 || block_size > sizeof(buf)) {
		fprintf(stderr,
			"%s: cannot pad to %d bytes (limit is %ld bytes)\n",
			argv[0], block_size, sizeof(buf));
		exit(1);
	    } /* if */
	    continue;
	} /* if */

	infp = fopen(argv[i], "r");
	if (!infp) {
	    perror("fopen");
	    goto cleanup;
	} /* if */
	sprintf(outname, "%s.exec", argv[i]);
	outfp = fopen(outname, "w");
	if (!outfp) {
	    perror("fopen");
	    goto cleanup;
	} /* if */

	/* read and inspect exec header: */

	if (fread(&exhdr, 1, sizeof(exhdr), infp) != sizeof(exhdr)) {
	    fprintf(stderr, "%s: fread() from %s: exec header short read\n",
		    argv[0], argv[i]);
	    goto cleanup;
	} /* if */

	if (!(exhdr.ex_f.f_flags & F_EXEC)) {
	    fprintf(stderr, "%s: %s is not in executable format\n",
		    argv[0], argv[i]);
	    goto cleanup;
	} /* if */

	if (exhdr.ex_f.f_opthdr != sizeof(exhdr.ex_o)) {
	    fprintf(stderr, "%s: %s has unexpected optional header size\n",
		    argv[0], argv[i]);
	    goto cleanup;
	} /* if */

	if (exhdr.ex_o.magic != OMAGIC) {
	    fprintf(stderr,
		    "%s: %s is not an OMAGIC file\n", argv[0], argv[i]);
	    goto cleanup;
	} /* if */

	fseek(infp, N_TXTOFF(exhdr.ex_f, exhdr.ex_o), SEEK_SET);

	tot_size = exhdr.ex_o.tsize + exhdr.ex_o.dsize;

	if (verbose) {
	    fprintf(stderr,
		    "%s: copying %d byte from %s to %s\n",
		    argv[0], tot_size, argv[i], outname);
	} /* if */

	for (size = 0; size < tot_size; size += nxfer) {
	    nxfer = tot_size - size;
	    if (nxfer > sizeof(buf)) {
		nxfer = sizeof(buf);
	    } /* if */
	    if (fread(buf, 1, nxfer, infp) != nxfer) {
		fprintf(stderr,
			"%s: fread() from %s: section short read\n",
			argv[0], argv[i]);
		goto cleanup;
	    } /* if */
	    if (fwrite(buf, 1, nxfer, outfp) != nxfer) {
		fprintf(stderr,
			"%s: fwrite() on %s: section short write\n",
			argv[0], argv[i]);
		goto cleanup;
	    } /* if */
	} /* for */

	pad = block_size - (tot_size % block_size);
	if (pad < block_size) {
	    fprintf(stderr, "%s: padding with %d bytes\n", argv[0], pad);
	    memset(buf, 0x00, pad);
	    if (fwrite(buf, 1, pad, outfp) != pad) {
		fprintf(stderr,
			"%s: fwrite() on %s: pad short write\n",
			argv[0], argv[i]);
		goto cleanup;
	    } /* if */
	} /* if */

      cleanup:
	if (infp) {
	    fclose(infp);
	} /* if */
	if (outfp) {
	    fclose(outfp);
	} /* if */
    } /* for */
    return 0;
} /* main */

			/*** end of mkboot.c ***/
